#include "gibbs_normal.h"
/* ------------------------------------------------------- */
/* Improved Gibbs sampler for a Mixture of Normals (burn)  */
/* ------------------------------------------------------- */
void gibbs_normal_burn(int burn, int k, int n, double *y, int *z, int *nj, double *mu,
		       double delta, double kappa, double alpha, double *beta,
		       double gg, double hh, double cte, double *y_sum, double *y2_sum){
  int h, j;
  double w[k], tau[k], sqrt_tau[k], temp_var, scale_weights, tmp_tau, sum_tau;

  /* ################################################## */
  /* ################ Starting Iterations ############# */
  /* ################################################## */
  for(h = 0; h < burn; h++){
    /* Setting to zero - at each iteration */
    scale_weights = 0.0;
    sum_tau = 0.0;
		/* Updating means, precisions, beta and, weights */
    for(j = 0; j < k; j++){
      tmp_tau = y2_sum[j] - 2.0*mu[j]*y_sum[j] + ((double) nj[j])*mu[j]*mu[j];
      tau[j] = rmygamma(alpha + 0.5*((double) nj[j]), *beta + 0.5*tmp_tau);
      sqrt_tau[j] = sqrt(tau[j]);
      temp_var = 1.0/(tau[j]*((double) nj[j]) + kappa);
      mu[j] = rnorm(temp_var*(tau[j]*y_sum[j] + cte),  sqrt(temp_var));
      w[j] = rmygamma(((double) nj[j]) + delta, 1.0); /* For the weights*/
      scale_weights += w[j];
      sum_tau += tau[j];  /* To update beta */
    }
    /* updating beta */
    *beta = rmygamma(gg + k*alpha, hh + sum_tau);
    /* Scaling the weights */
    for(j = 0; j < k; j++){
			w[j] /= scale_weights;
      nj[j] = 0;
      y_sum[j] = 0.0;
      y2_sum[j] = 0.0;      
    }
    /* Updating allocations */
    sample_allocations_burn(k, n, z, y, w, mu, tau, sqrt_tau, nj, y_sum, y2_sum);
  }
}

/* --------------------------------------------------------------- */
/* Improved Gibbs sampler for a Mixture of Normals (r.v. to store) */
/* --------------------------------------------------------------- */
void gibbs_normal(int m, int k, int n, double *y, int *z, int *nj, double *mu, double delta,
		  double mu0, double kappa, double alpha, double *beta, double gg, double hh,
		  double cte, double *y_sum, double *y2_sum, FILE *ffile_z, FILE *ffile_pz,
		  FILE *ffile_w, FILE *ffile_mu, FILE *ffile_sigma2, FILE *ffile_log_post){
	     
  int h, j;
  double w[k], tau[k], sqrt_tau[k], temp_var, scale_weights, tmp_tau, sum_tau, sum_log_tau, 
	       tmp_log, log_like, log_mu, log_tau, log_zw, log_beta, log_post;
  
  /* ################################################## */
  /* ################ Starting Iterations ############# */
  /* ################################################## */
  for(h = 0; h < m; h++){
    
    /* Setting to zero - at each iteration */
    scale_weights = 0.0; 
    sum_tau = 0.0;		
		/* for the log posterior */
		sum_log_tau = 0.0;
		log_like = 0.0;
		log_mu = 0.0;
		log_tau = 0.0;
    log_zw = 0.0;
		log_beta = 0.0;
    log_post = 0.0;
    
    /* Updating means, precisions, beta and, weights */    
    for(j = 0; j < k; j++){
      tmp_tau = y2_sum[j] - 2.0*mu[j]*y_sum[j] + ((double) nj[j])*mu[j]*mu[j];
      tau[j] = rmygamma(alpha + 0.5*((double) nj[j]), *beta + 0.5*tmp_tau);
      sqrt_tau[j] = sqrt(tau[j]);
      temp_var = 1.0/(tau[j]*((double) nj[j]) + kappa);
      mu[j] = rnorm(temp_var*(tau[j]*y_sum[j] + cte),  sqrt(temp_var));
      w[j] = rmygamma(((double) nj[j]) + delta, 1.0); /* For the weights*/
      scale_weights += w[j];
      sum_tau += tau[j];  /* To update beta */
      fprintf(ffile_mu, "%12.16f \t", mu[j]);
			fprintf(ffile_sigma2, "%12.16f \t", 1.0/tau[j]);       
    }
    fprintf(ffile_mu, "\n");
    fprintf(ffile_sigma2, "\n");        
		/* updating beta */
    *beta = rmygamma(gg + k*alpha, hh + sum_tau);     
    /* Scaling the weights and calculating bits of the log posterior*/
    for(j = 0; j < k; j++){
			w[j] /= scale_weights;
			fprintf(ffile_w, "%12.16f \t", w[j]);					
			/* First bit of the log posterior */
			tmp_log = log(tau[j]);
			tmp_tau = y2_sum[j] - 2.0*mu[j]*y_sum[j] + ((double) nj[j])*mu[j]*mu[j];			
			log_like += ((double) nj[j])*tmp_log - tau[j]*tmp_tau;  			
			log_mu += (mu[j] - mu0)*(mu[j] - mu0);
			sum_log_tau += tmp_log;		
			log_zw += (((double) nj[j]) + delta - 1.0)*log(w[j]);			
			/* setting to zero basic statistics */
      nj[j] = 0;
      y_sum[j] = 0.0;
      y2_sum[j] = 0.0;            
    }
    fprintf(ffile_w, "\n");    
		
		/* Second bit of the log posterior */
		log_like /= 2.0;
		log_mu *= -kappa/2.0;
		log_tau += ((double) k)*alpha*log(*beta) + (alpha - 1.0)*sum_log_tau;
		log_tau -= (*beta)*sum_tau;
		log_beta += (gg - 1.0)*log(*beta) - hh*(*beta);		
		
		/* log posterior */
		log_post += log_like + log_mu + log_tau + log_zw + log_beta;
		
		fprintf(ffile_log_post, "%12.16f \n", log_post);

    /* Updating allocations */
    sample_allocations(k, n, z, y, w, mu, tau, sqrt_tau, nj, y_sum, y2_sum, ffile_z, ffile_pz);
  }  
}